<?php /* vim: set noet ts=4 sw=4: : */

/* Generates an RSS/RDF feed for a set of bugs
 * based on search criteria as provided.
 *
 * Search code borrowed from /search.php (As of Revision: 1.82)
 * and accepts the same parameters.
 *
 * When changes are made to that API,
 * they should be reflected here for consistency
 *
 * borrowed from php-bugs-web, implementation by Sara Golemon <pollita@php.net>
 * ported by Gregory Beaver <cellog@php.net>
 */

/* Maximum number of bugs to return */
define ('MAX_BUGS_RETURN', 150);

/**
 * Obtain common includes
 */
require_once '../include/functions.inc';

$errors = array();
$warnings = array();
$order_options = array(
    ''             => 'relevance',
    'id'           => 'ID',
    'ts1'          => 'date',
    'package_name' => 'package',
    'bug_type'     => 'bug_type',
    'status'       => 'status',
    'package_version'  => 'package_version',
    'php_version'  => 'php_version',
    'php_os'       => 'os',
    'sdesc'        => 'summary',
    'assign'       => 'assignment',
);

define('BOOLEAN_SEARCH', @intval($_GET['boolean']));

/*
* need to move this to DB eventually...
*/
$mysql4 = false;
if ($dbh->phptype == 'mysql') {
    $mysql4 = version_compare(mysql_get_server_info(), '4.0.0', 'ge');
} else {
    $mysql4 = version_compare(mysqli_get_server_version($dbh->connection), '4.0.0', 'ge');

}

if ($mysql4) {
    $query = 'SELECT SQL_CALC_FOUND_ROWS';
} else {
    $query = 'SELECT';
}

$query .= ' bugdb.*, TO_DAYS(NOW())-TO_DAYS(bugdb.ts2) AS unchanged'
        . ' FROM bugdb';

if (!empty($site) || !empty($_GET['maintain']) || !empty($_GET['handle'])) {
    $query .= ' LEFT JOIN packages ON packages.name = bugdb.package_name';
}

if (!empty($_GET['maintain']) || !empty($_GET['handle'])) {
    $query .= ' LEFT JOIN maintains ON packages.id = maintains.package';
    $query .= ' AND maintains.handle = ';
    if (!empty($_GET['maintain'])) {
        $query .= $dbh->quoteSmart($_GET['maintain']);
    } else {
        $query .= $dbh->quoteSmart($_GET['handle']);
    }
}

if (empty($_GET['package_name']) || !is_array($_GET['package_name'])) {
    $_GET['package_name'] = array();
    $where_clause = ' WHERE bugdb.registered=1';
} else {
    $where_clause = ' WHERE bugdb.registered=1 AND bugdb.package_name';
    if (count($_GET['package_name']) > 1) {
        $where_clause .= " IN ('"
                       . join("', '", escapeSQL($_GET['package_name']))
                       . "')";
    } else {
        $where_clause .= ' = '
                           . $dbh->quoteSmart($_GET['package_name'][0]);
    }
}

if (empty($_GET['package_nname']) || !is_array($_GET['package_nname'])) {
    $_GET['package_nname'] = array();
} else {
    $where_clause .= ' AND bugdb.package_name';
    if (count($_GET['package_nname']) > 1) {
        $where_clause .= " NOT IN ('"
                       . join("', '", escapeSQL($_GET['package_nname']))
                       . "')";
    } else {
        $where_clause .= ' <> '
                       . $dbh->quoteSmart($_GET['package_nname'][0]);
    }
}

/*
 * Ensure status is valid and tweak search clause
 * to treat assigned, analyzed, critical and verified bugs as open
 */
if (empty($_GET['status'])) {
    $status = 'Open';
} else {
    $status = $_GET['status'];
}
switch ($status) {
    case 'All':
        break;
    case 'Closed':
    case 'Duplicate':
    case 'Critical':
    case 'Assigned':
    case 'Analyzed':
    case 'Verified':
    case 'Suspended':
    case 'Wont fix':
    case 'No Feedback':
    case 'Feedback':
    case 'Bogus':
        $where_clause .= " AND bugdb.status='$status'";
        break;
    case 'Old Feedback':
        $where_clause .= " AND bugdb.status='Feedback'" .
                         ' AND TO_DAYS(NOW())-TO_DAYS(bugdb.ts2) > 60';
        break;
    case 'Fresh':
        $where_clause .= ' AND bugdb.status NOT IN' .
                         " ('Closed', 'Duplicate', 'Bogus')" .
                         ' AND TO_DAYS(NOW())-TO_DAYS(bugdb.ts2) < 30';
        break;
    case 'Stale':
        $where_clause .= ' AND bugdb.status NOT IN' .
                         " ('Closed', 'Duplicate', 'Bogus')" .
                         ' AND TO_DAYS(NOW())-TO_DAYS(bugdb.ts2) > 30';
        break;
    case 'Not Assigned':
        $where_clause .= ' AND bugdb.status NOT IN' .
                         " ('Closed', 'Duplicate', 'Bogus', 'Assigned'," .
                         " 'Wont Fix', 'Suspended')";
        break;
    // Closed Reports Since Last Release
    case 'CRSLR':
        if (!isset($_GET['package_name']) || count($_GET['package_name']) > 1) {
            // Act as ALL
            break;
        }

        // Fetch the last release date
        include_once 'pear-database-package.php';
        $releaseDate = package::getRecent(1, rinse($_GET['package_name'][0]));
        if (PEAR::isError($releaseDate)) {
            break;
        }

        $where_clause .= ' AND bugdb.status IN' .
                         " ('Closed', 'Duplicate', 'Bogus', 'Wont Fix', 'Suspended')
                           AND (UNIX_TIMESTAMP('" . $releaseDate[0]['releasedate'] . "') < UNIX_TIMESTAMP(bugdb.ts2))
                         ";
        break;
    case 'Open':
    default:
        $where_clause .= " AND bugdb.status IN ('Open', 'Assigned'," .
                         " 'Analyzed', 'Critical', 'Verified')";
    case 'OpenFeedback':
    default:
        $where_clause .= " AND bugdb.status IN ('Open', 'Assigned'," .
                         " 'Analyzed', 'Critical', 'Verified', 'Feedback')";
}

if (empty($_GET['search_for'])) {
    $search_for = '';
} else {
    $search_for = htmlspecialchars($_GET['search_for']);
    list($sql_search, $ignored) = format_search_string($search_for);
    $where_clause .= $sql_search;
    if (count($ignored) > 0 ) {
        $warnings[] = 'The following words were ignored: ' .
                rinse(implode(', ', array_unique($ignored)));
    }
}

if (empty($_GET['bug_type']) || $_GET['bug_type'] == 'All') {
    $bug_type = '';
} else {
    $bug_type = $_GET['bug_type'];
    if ($bug_type == 'Bugs') {
        $where_clause .= ' AND (bugdb.bug_type = "Bug" OR bugdb.bug_type="Documentation Problem")';
    } else {
        $where_clause .= ' AND bugdb.bug_type = ' . $dbh->quoteSmart($bug_type);
    }
}

if (empty($_GET['bug_age']) || !(int)$_GET['bug_age']) {
    $bug_age = 0;
} else {
    $bug_age = (int)$_GET['bug_age'];
    $where_clause .= ' AND bugdb.ts1 >= '
                   . " DATE_SUB(NOW(), INTERVAL $bug_age DAY)";
}

if (empty($_GET['bug_updated']) || !(int)$_GET['bug_updated']) {
    $bug_updated = 0;
} else {
    $bug_updated = (int)$_GET['bug_updated'];
    $where_clause .= ' AND bugdb.ts2 >= '
                   . " DATE_SUB(NOW(), INTERVAL $bug_updated DAY)";
}

if (empty($_GET['php_os'])) {
    $php_os = '';
} else {
    $php_os = $_GET['php_os'];
    $where_clause .= " AND bugdb.php_os LIKE '%"
                   . $dbh->escapeSimple($php_os) . "%'";
}

if (empty($_GET['phpver'])) {
    $phpver = '';
} else {
    $phpver = $_GET['phpver'];
    $where_clause .= " AND bugdb.php_version LIKE '"
                   . $dbh->escapeSimple($phpver) . "%'";
}

if (empty($_GET['packagever'])) {
    $packagever = '';
} else {
    $packagever = $_GET['packagever'];
    $where_clause .= " AND bugdb.package_version LIKE '"
                   . $dbh->escapeSimple($packagever) . "%'";
}

if (empty($_GET['handle'])) {
    $handle = '';
    if (empty($_GET['assign'])) {
        $assign = '';
    } else {
        $assign = $_GET['assign'];
        $where_clause .= ' AND bugdb.assign = '
                       . $dbh->quoteSmart($assign);
    }
    if (empty($_GET['maintain'])) {
        $maintain = '';
    } else {
        $maintain = $_GET['maintain'];
        $where_clause .= ' AND maintains.handle = '
                       . $dbh->quoteSmart($maintain);
    }
} else {
    $handle = $_GET['handle'];
    $where_clause .= ' AND (maintains.handle = '
                   . $dbh->quoteSmart($handle)
                   . ' OR bugdb.assign = '
                   . $dbh->quoteSmart($handle). ')';
}

if (empty($_GET['author_email'])) {
    $author_email = '';
} else {
    $author_email = $_GET['author_email'];
    $qae = $dbh->quoteSmart($author_email);
    $where_clause .= ' AND (bugdb.email = '
                   . $qae . ' OR bugdb.handle=' . $qae . ')';
}

$where_clause .= ' AND (packages.package_type = '
               . $dbh->quoteSmart($site);

if ($pseudo = array_intersect($pseudo_pkgs, $_GET['package_name'])) {
    $where_clause .= " OR bugdb.package_name";
    if (count($pseudo) > 1) {
        $where_clause .= " IN ('"
                       . join("', '", escapeSQL($pseudo)) . "')";
    } else {
        $where_clause .= " = '" . implode('', escapeSQL($pseudo)) . "'";
    }
} else {
    $where_clause .= " OR bugdb.package_name IN ('"
                   . join("', '", escapeSQL($pseudo_pkgs)) . "')";
}

$where_clause .= ')';

$query .= $where_clause;

if (isset($_GET['direction']) && $_GET['direction'] != 'DESC') {
    $direction = 'ASC';
} else {
    $direction = 'DESC';
}

if (empty($_GET['order_by']) ||
    !array_key_exists($_GET['order_by'], $order_options))
{
    $order_by = 'id';
} else {
    $order_by = $_GET['order_by'];
}

if (empty($_GET['reorder_by']) ||
    !array_key_exists($_GET['reorder_by'], $order_options))
{
    $reorder_by = '';
} else {
    $reorder_by = $_GET['reorder_by'];
    if ($order_by == $reorder_by) {
        $direction = $direction == 'ASC' ? 'DESC' : 'ASC';
    } else {
        $direction = 'ASC';
        $order_by = $reorder_by;
    }
}

$query .= ' ORDER BY ' . $order_by . ' ' . $direction;

// if status Feedback then sort also after last updated time.
if ($status == 'Feedback') {
    $query .= ', bugdb.ts2 ' . $direction;
}

if (empty($_GET['limit']) || !(int)$_GET['limit']) {
    if (!empty($_GET['limit']) && $_GET['limit'] == 'All') {
        $limit = 'All';
    } else {
        $limit = MAX_BUGS_RETURN;
        $query .= " LIMIT 0, $limit";
    }
} else {
    $limit  = (int)$_GET['limit'];
    $query .= " LIMIT 0, $limit";
}

if (stristr($query, ';')) {
    die('BAD HACKER!! No database cracking for you today!');
} else {
    $res  = $dbh->query($query);
    $rows =  $res->numRows();

    $total_rows = $dbh->getOne('SELECT FOUND_ROWS()');
    if ($total_rows > $rows) {
        $warnings[] = 'The search was too general, only ' . MAX_BUGS_RETURN .
            ' bugs will be returned';
    }
}


header('Content-type: text/xml');

echo '<?xml version="1.0" encoding="UTF-8"?>
<rdf:RDF
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns="http://purl.org/rss/1.0/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
>
  <channel rdf:about="http://bugs.php.net/">
    <title>PHP Bugs Search Results</title>
    <link>http://bugs.php.net/rss/search.php</link>
    <description>$descr</description>
    <items>
     <rdf:Seq>

';

if ($total_rows > 0) {
	$i = 0;
	$items = array();
	while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
	    $i++;
		echo "      <rdf:li rdf:resource=\"http://bugs.php.net/{$row['id']}\" />\n";
	    $items[$i] = "    <item>\n";
	    $items[$i] .= '      <title>' . utf8_encode(htmlspecialchars('[' . $row['status'] . '] ' . $row['sdesc'])) . "</title>\n";
	    $items[$i] .= "      <link>http://bugs.php.net/{$row['id']}</link>\n";
	    $items[$i] .= '      <description>' . utf8_encode(htmlspecialchars($row['ldesc'])) . "</description>\n";
	    if (!$row['unchanged']) {
    	    $items[$i] .= '      <dc:date>' . date('Y-m-d',strtotime($row['ts1'])) . "</dc:date>\n";
    	    $items[$i] .= '      <dc:time>' . date('H:i:s',strtotime($row['ts1'])) . "</dc:time>\n";
	    } else {
    	    $items[$i] .= '      <dc:date>' . date('Y-m-d',strtotime($row['ts2'])) . "</dc:date>\n";
    	    $items[$i] .= '      <dc:time>' . date('H:i:s',strtotime($row['ts2'])) . "</dc:time>\n";
	    }
	    $items[$i] .= '      <dc:creator>' . utf8_encode(htmlspecialchars(spam_protect($row['email']))) . "</dc:creator>\n";
	    $items[$i] .= '      <dc:subject>' .
	       utf8_encode(htmlspecialchars($row['package_name'])) . ' ' .
	       utf8_encode(htmlspecialchars($row['bug_type'])) . "</dc:subject>\n";
	    $items[$i] .= "    </item>\n";
	}
	$items = implode('', $items);
} else {
	$warnings[] = "No bugs matched your criteria";
}

echo '
     </rdf:Seq>
    </items>
  </channel>

  <image rdf:about="http://pear.php.net/gifs/pearsmall.gif">
    <title>PEAR Bugs</title>
    <url>http://pear.php.net/gifs/pearsmall.gif</url>
    <link>http://pear.php.net/bugs</link>
  </image>

', $items;
?>
</rdf:RDF>
<?php
if (count($warnings) > 0) {
	echo "<!--\n\n";
	echo "The following warnings occured during your request:\n\n";
	foreach($warnings as $warning) {
		echo utf8_encode(htmlspecialchars('* ' . $warning)) . "\n";
	}
	echo "-->\n";
}
